package org.neo4j.tutor.search;

import org.neo4j.api.core.RelationshipType;

public enum MyRelationshipTypes implements RelationshipType
{
	// relationship between two keywords 
	TRANSLATION,
	// relationship between a keyword and a document
	HAS_KEYWORD,
	// relationship between a language node and keywords
	KEYWORD,
	// relationship between reference node and language nodes
	KEYWORD_LANGUAGE,
	// relationship between reference node and document nodes
	DOCUMENT
}
