package org.neo4j.tutor.security;

import org.neo4j.api.core.RelationshipType;

public enum MyRelationshipTypes implements RelationshipType
{
	// relationship between a parent file and child file
	FILE,
	// relationship between a file and a user/group
	ACCESS,
	// relationship between a user/group and group
	MEMBER_OF
}
