package org.neo4j.tutor.tree;

import org.neo4j.api.core.RelationshipType;

public enum MyRelationshipTypes implements RelationshipType
{
	// relationship between a parent file and child file
	FILE
}
