package org.neo4j.tutor.security;

import org.neo4j.impl.event.Event;

public class SecurityEvent extends Event
{
	public static final SecurityEvent ACCESS = new SecurityEvent( "ACCESS" );
	
	private SecurityEvent( String name )
	{
		super( name );
	}
}
